UNIT Main;
{+H
---------------------------------------------------------------------------
  File        - MAIN.PAS
  Purpose     - Print 1, 2 or 3 pages of text per side of paper.
  Remarks     - Based on FourPage by Mark Erbaugh.
                Enhanced by Keith S. Brown
                With the right kind of printer you can "flip" and reinsert
                printed pages to print on both sides. The "flip" technique
                works best with a local printer.
                With a busy networked printer you will have to reassemble
                the pages in the correct order.
                Modified to automatically print the DFM file (if any) associated
                with the Delphi source file.
                With 3 pages per side
                  text is clipped after the 79th character
                With 2 pages per side
                  text is clipped after the 90th character
                With 1 page per side
                  text is clipped after the 139th character.
                
  Copyright (c) 1995 - Microcomputer Enhancement
  Revised     - 1999.0909 (KSB) Reformatted.
              - 1999.0910 (KSB) Simplified logic, removed extraneous code,
                added ability to print (delphi) DFM files.
---------------------------------------------------------------------------}
INTERFACE
USES
  Classes,  Controls, Dialogs,  Forms,    Graphics, Messages, Printers, StdCtrls,
  SysUtils, WinProcs, WinTypes;

TYPE
  TfrmFourPage= CLASS(TForm)
    dlgOpen        : TOpenDialog;
    dlgPrntSetup   : TPrinterSetupDialog;
    btnClose       : TButton;
    btnPrint       : TButton;
    btnPrntSetup   : TButton;
    rbtn1          : TRadioButton;
    rbtn2          : TRadioButton;
    rbtn3          : TRadioButton;
    PROCEDURE   BtnPrintClick(aoSender: TObject);
    PROCEDURE   BtnPrintSetupClick(aoSender: TObject);
    PROCEDURE   BtnCloseClick(aoSender: TObject);
  END {CLASS};

VAR
  frmFourPage: TfrmFourPage;
              {=================================================}
IMPLEMENTATION
{$R *.DFM}


{}PROCEDURE   TfrmFourPage.BtnCloseClick(aoSender: TObject);
{+O
---------------------------------------------------------------------------
  Purpose     - Close this form and thus the application
---------------------------------------------------------------------------}
  BEGIN
    Close;
{}END {BtnCloseClick};




{}PROCEDURE   TfrmFourPage.BtnPrintClick(aoSender: TObject);
{+O
---------------------------------------------------------------------------
  Purpose     -
---------------------------------------------------------------------------}
  CONST
    csSecond  = 'Insert page(s) for second side';
  VAR
    lbHavePrinted  : BOOLEAN;
    lbIsPrint      : BOOLEAN;
    liAdjPhysWidth : INTEGER;
    liCharsPerLine : INTEGER;
    liLinesPerPage : INTEGER;
    liOffset       : INTEGER;
    liPageNum      : INTEGER;
    liPagesPerPage : INTEGER;
    liPageWidth    : INTEGER;          {width of a single "virtual" page}
    liRowHeight    : INTEGER;
    liRowOffset    : INTEGER;
    lrPhysSize     : TPoint;
    lrPrintOffset  : TPoint;
    lsFileName     : STRING;
    lsDateTime     : STRING;
    ltxt           : TextFile;


{}{}PROCEDURE DoLayout;
    VAR
      li      : INTEGER;
      liCol   : INTEGER;
      liJ     : INTEGER;
      liRow   : INTEGER;
      ls      : STRING;
    BEGIN
      liCol := liOffset;
      Inc(liPageNum, 1);
      IF lbIsPrint THEN BEGIN
        ls := LowerCase(Format('Page %d: %s @ %s', [liPageNum, lsFileName, lsDateTime]));
        Printer.Canvas.TextOut(liCol, 0, ls);
      END {IF};

      FOR li := 1 TO liPagesPerPage DO BEGIN
        liRow := 2 * liRowHeight;
        FOR liJ := 3 TO liLinesPerPage DO BEGIN
          IF Eof(ltxt) THEN BEGIN
            Exit;
          END {IF};

          ReadLn(ltxt, ls);
          IF lbIsPrint THEN BEGIN
            ls := Copy(ls, 1, liCharsPerLine);
            Printer.Canvas.TextOut(liCol, liRow, ls);
            lbHavePrinted := TRUE;
            Inc(liRow, liRowHeight);
          END {IF};
        END {FOR};

        Inc(liCol, liPageWidth);
      END {FOR};
{}{}END {DoLayout};


{}{}PROCEDURE OpenSourceFile(VAR F:TextFile; asFileName:STRING);
    VAR
      liChars      : INTEGER;
      loDFMstream  : TFileStream;
      loTXTstream  : TFileStream;
      lsTmpFileName: STRING;
    BEGIN
      IF ExtractFileExt(asFileName) = '.DFM' THEN BEGIN
        SetLength(lsTmpFileName,255);
        liChars := GetTempPath(255,PChar(lsTmpFileName));
        IF liChars <> 0 THEN BEGIN
          lsTmpFileName := Copy(lsTmpFileName,1,liChars) + 'X.TXT';
        END ELSE BEGIN
          lsTmpFileName := 'X.TXT';
        END {IF};

        loDFMstream := TFileStream.Create(asFileName,fmOpenRead OR fmShareCompat);
        TRY
          loTXTstream := TFileStream.Create(lsTmpFileName,fmCreate OR fmShareExclusive);
          TRY
            ObjectResourceToText(loDFMstream,loTXTstream);
          FINALLY
            loTXTstream.Free;
          END {TRY};
        FINALLY
          loDFMstream.Free;
        END {TRY};
      END ELSE BEGIN
        lsTmpFileName := asFileName;
      END {IF};

      AssignFile(F, lsTmpFileName);
      Reset(F);
{}{}END {OpenSourceFile};


CONST
  cbODD       = TRUE;
  cbEVEN      = FALSE;

{}{}PROCEDURE Pages(CONST abIsOddNoPage:BOOLEAN);
    BEGIN
      lsDateTime := DateTimeToStr(FileDateToDateTime(FileAge(lsFileName)));
      OpenSourceFile(ltxt,lsFileName);
      lbHavePrinted := FALSE;
      liPageNum := 0;
      lbIsPrint := abIsOddNoPage;
      REPEAT
        IF lbHavePrinted THEN BEGIN
          Printer.NewPage;
        END {IF};

        lbHavePrinted := FALSE;
        DoLayout;
        lbIsPrint := NOT lbIsPrint
      UNTIL Eof(ltxt);

      IF abIsOddNoPage AND lbHavePrinted THEN BEGIN
        Printer.NewPage;
      END {IF};

      CloseFile(ltxt);
{}{}END {Pages};


  VAR
    liFiles   : INTEGER;
  BEGIN
    IF dlgOpen.Execute THEN BEGIN
      WITH Printer DO BEGIN
              {select number of "virtual" pages per page}
        IF rbtn1.Checked THEN BEGIN
          Orientation := poPortrait;
          liPagesPerPage := 1;
          Canvas.Font.Size := 7;
        END ELSE
        IF rbtn2.Checked THEN BEGIN
          Orientation := poLandscape;
          liPagesPerPage := 2;
          Canvas.Font.Size := 7;
        END ELSE
        IF rbtn3.Checked THEN BEGIN
          Orientation := poLandscape;
          liPagesPerPage := 3;
          Canvas.Font.Size := 5;
        END {IF};

              {Compute lines per page}
        Canvas.Font.Name := 'Courier New';
        liRowHeight   := Canvas.TextHeight('0');
        liLinesPerPage:= Printer.PageHeight DIV liRowHeight;
              {...retrieve the physical page size, copy it to lrPhysSize.}
        Escape(Handle, GETPHYSPAGESIZE, 0, NIL, @lrPhysSize);
              {...retrieve the offset from the upper-left corner of the physical
                  page where the actual printing or drawing begins.}
        Escape(Handle, GETPRINTINGOFFSET, 0, NIL, @lrPrintOffset);
        liRowOffset := lrPhysSize.x - Printer.PageWidth - lrPrintOffset.x;
        IF liRowOffset > lrPrintOffset.x THEN BEGIN
          liOffset := liRowOffset - lrPrintOffset.x;
          liAdjPhysWidth := Printer.PageWidth - liOffset;
        END ELSE BEGIN
          liOffset := 0;
          liAdjPhysWidth := Printer.PageWidth - (lrPrintOffset.x - liRowOffset);
        END {IF};

        liPageWidth   := liAdjPhysWidth DIV liPagesPerPage;
        liCharsPerLine:= (liPageWidth DIV Printer.Canvas.TextWidth('0')) - 2;
              {...add the corresponding DFM file to the list if it exists...}
        FOR liFiles := 0 TO dlgOpen.Files.Count-1 DO BEGIN
          lsFileName := dlgOpen.Files.Strings[liFiles];
          IF UpperCase(ExtractFileExt(lsFileName)) = '.PAS' THEN BEGIN
            lsFileName := ChangeFileExt(lsFileName,'.DFM');
            IF FileExists(lsFileName) THEN BEGIN
              dlgOpen.Files.Append(lsFileName);
            END {IF};
          END {IF};
        END {FOR};

              {Print the selected files}
        BeginDoc;
        FOR liFiles := 0 TO dlgOpen.Files.Count - 1 DO BEGIN
          lsFileName := dlgOpen.Files.Strings[liFiles];
          Pages(cbODD);
        END {FOR};

        IF MessageDlg(csSecond, mtConfirmation, [mbOK, mbCancel], 0) = mrOK THEN BEGIN
          FOR liFiles := 0 TO dlgOpen.Files.Count - 1 DO BEGIN
            IF liFiles > 0 THEN BEGIN
              Printer.NewPage;
            END {IF};

            lsFileName := dlgOpen.Files.Strings[liFiles];
            Pages(cbEVEN);
          END {FOR};
        END ELSE BEGIN
          Printer.Abort;
          WinProcs.AbortDoc(Printer.Canvas.Handle);
        END {IF};

        EndDoc;
      END {IF};
    END {WITH};
{}END {BtnPrintClick};




{}PROCEDURE   TfrmFourPage.BtnPrintSetupClick(aoSender: TObject);
{+O
---------------------------------------------------------------------------
  Purpose     - Configure the printer.
---------------------------------------------------------------------------}
  BEGIN
    dlgPrntSetup.Execute;
{}END {BtnPrintSetupClick};


END.
